@@ -1,6 +1,6 @@
 /*
  *  git2r, R bindings to the libgit2 library.
- *  Copyright (C) 2013-2018 The git2r contributors
+ *  Copyright (C) 2013-2020 The git2r contributors
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License, version 2,
@@ -16,9 +16,11 @@
  *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
+#include <R_ext/Visibility.h>
 #include <git2.h>
 
 #include "git2r_arg.h"
+#include "git2r_deprecated.h"
 #include "git2r_error.h"
 #include "git2r_oid.h"
 #include "git2r_repository.h"
@@ -32,7 +34,10 @@
  * @return Integer vector of length two with the values ahead and
  * behind.
  */
-SEXP git2r_graph_ahead_behind(SEXP local, SEXP upstream)
+SEXP attribute_hidden
+git2r_graph_ahead_behind(
+    SEXP local,
+    SEXP upstream)
 {
     size_t ahead, behind;
     int error, nprotect = 0;
@@ -79,7 +84,7 @@ cleanup:
         UNPROTECT(nprotect);
 
     if (error)
-        git2r_error(__func__, giterr_last(), NULL, NULL);
+        git2r_error(__func__, GIT2R_ERROR_LAST(), NULL, NULL);
 
     return result;
 }
@@ -91,7 +96,10 @@ cleanup:
  * @param ancestor A potential ancestor commit.
  * @return TRUE or FALSE
  */
-SEXP git2r_graph_descendant_of(SEXP commit, SEXP ancestor)
+SEXP attribute_hidden
+git2r_graph_descendant_of(
+    SEXP commit,
+    SEXP ancestor)
 {
     int error, descendant_of = 0;
     SEXP commit_repo, commit_sha;
@@ -129,7 +137,7 @@ cleanup:
     git_repository_free(repository);
 
     if (error)
-        git2r_error(__func__, giterr_last(), NULL, NULL);
+        git2r_error(__func__, GIT2R_ERROR_LAST(), NULL, NULL);
 
     return Rf_ScalarLogical(descendant_of);
 }
