@@ -1,5 +1,5 @@
 ## git2r, R bindings to the libgit2 library.
-## Copyright (C) 2013-2018 The git2r contributors
+## Copyright (C) 2013-2019 The git2r contributors
 ##
 ## This program is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License, version 2,
@@ -20,43 +20,66 @@ library("git2r")
 sessionInfo()
 
 ## Create a directory in tempdir
-path <- tempfile(pattern="git2r-")
+path <- tempfile(pattern = "git2r-")
 dir.create(path)
 
 ## Initialize a repository
 repo <- init(path)
 
 ## Config repository
-cfg <- config(repo, user.name="Alice", user.email="alice@example.org")
+cfg <- config(repo, user.name = "Alice", user.email = "alice@example.org")
 
 ## Check configuration
+stopifnot(identical(print(cfg), cfg))
 stopifnot("local" %in% names(cfg))
 stopifnot("user.name" %in% names(cfg$local))
 stopifnot(identical(cfg$local$user.name, "Alice"))
 stopifnot(identical(cfg$local$user.email, "alice@example.org"))
 
+## Check that config fails for non-character entry.
+tools::assertError(config(repo, test = 5))
+
+## Check config method with missing repo argument
+wd <- setwd(path)
+cfg <- config(user.name = "Alice", user.email = "alice@example.org")
+stopifnot("local" %in% names(cfg))
+stopifnot("user.name" %in% names(cfg$local))
+stopifnot(identical(cfg$local$user.name, "Alice"))
+stopifnot(identical(cfg$local$user.email, "alice@example.org"))
+stopifnot(identical(git_config_files(repo = repo)$local,
+                    git_config_files(repo = NULL)$local))
+stopifnot(identical(git_config_files(repo = repo)$local,
+                    git_config_files(repo = repo$path)$local))
+if (!is.null(wd))
+    setwd(wd)
+
 ## Delete entries
-cfg <- config(repo, user.name=NULL, user.email=NULL)
+cfg <- config(repo, user.name = NULL, user.email = NULL)
 
 ## Check configuration
 stopifnot(is.null(cfg$local$user.name))
 stopifnot(is.null(cfg$local$user.email))
 
 ## Supply values as objects
-user.name <- "Alice"
-user.email <- "alice@example.org"
-cfg <- config(repo, user.name=user.name, user.email="alice@example.org")
-stopifnot(identical(cfg$local$user.name, user.name))
+user_name <- "Alice"
+user_email <- "alice@example.org"
+cfg <- config(repo, user.name = user_name, user.email = "alice@example.org")
+stopifnot(identical(cfg$local$user.name, user_name))
 stopifnot(identical(cfg$local$user.email, "alice@example.org"))
-cfg <- config(repo, user.name="Alice", user.email=user.email)
+cfg <- config(repo, user.name = "Alice", user.email = user_email)
 stopifnot(identical(cfg$local$user.name, "Alice"))
-stopifnot(identical(cfg$local$user.email, user.email))
+stopifnot(identical(cfg$local$user.email, user_email))
 
 ## Check git config files
 cfg <- git_config_files(repo)
-stopifnot(identical(length(cfg), 4L))
-stopifnot(identical(names(cfg), c("system", "xdg", "global", "local")))
-stopifnot(!is.na(cfg$local))
+stopifnot(identical(nrow(cfg), 4L))
+stopifnot(identical(names(cfg), c("file", "path")))
+stopifnot(identical(cfg$file, c("system", "xdg", "global", "local")))
+stopifnot(!is.na(cfg$path[4]))
+
+## Check that the local config file is NA for an invalid repo
+## argument.
+stopifnot(is.na(git_config_files(5)$local))
 
 ## Check location of .gitconfig on Windows
 if (identical(Sys.getenv("APPVEYOR"), "True")) {
@@ -99,4 +122,4 @@ if (identical(Sys.getenv("APPVEYOR"), "True")) {
 }
 
 ## Cleanup
-unlink(path, recursive=TRUE)
+unlink(path, recursive = TRUE)
