@@ -1,5 +1,5 @@
 ## git2r, R bindings to the libgit2 library.
-## Copyright (C) 2013-2018 The git2r contributors
+## Copyright (C) 2013-2019 The git2r contributors
 ##
 ## This program is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License, version 2,
@@ -20,12 +20,12 @@ library("git2r")
 sessionInfo()
 
 ## Create a directory in tempdir
-path <- tempfile(pattern="git2r-")
+path <- tempfile(pattern = "git2r-")
 dir.create(path)
 
 ## Initialize a repository
 repo <- init(path)
-config(repo, user.name="Alice", user.email="alice@example.org")
+config(repo, user.name = "Alice", user.email = "alice@example.org")
 
 ## Create a file
 writeLines("Hello world!", file.path(path, "test.txt"))
@@ -39,12 +39,13 @@ stopifnot(identical(.Call(git2r:::git2r_reference_dwim, repo, "")$name,
                     "refs/heads/master"))
 stopifnot(identical(.Call(git2r:::git2r_reference_dwim, repo, "master")$name,
                     "refs/heads/master"))
-stopifnot(identical(.Call(git2r:::git2r_reference_dwim, repo, "refs/heads/master")$name,
-                    "refs/heads/master"))
+stopifnot(identical(
+    .Call(git2r:::git2r_reference_dwim, repo, "refs/heads/master")$name,
+    "refs/heads/master"))
 
 ## print reference
 r <- .Call(git2r:::git2r_reference_dwim, repo, "refs/heads/master")
 stopifnot(identical(print(r), r))
 
 ## Cleanup
-unlink(path, recursive=TRUE)
+unlink(path, recursive = TRUE)
